package com.example.tdl.common;

import com.example.tdl.domain.vo.WarehouseDetailVo;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class WarehouseExcel {
    private Logger logger = LoggerFactory.getLogger(WarehouseExcel.class);

    //导出的文件名
    private String fileName;

    private String userName;

    private HttpServletResponse response;

    private WarehouseDetailVo warehouseDetailVo;

    public WarehouseExcel(String fileName, WarehouseDetailVo warehouseDetailVo ,String userName, HttpServletResponse response) {
        this.userName = userName;
        this.fileName = fileName;
        this.response = response;
        this.warehouseDetailVo=warehouseDetailVo;
    }

    /*
    * 导出数据
    * */
    public HSSFWorkbook downloadExcel() throws Exception {
        // 创建工作簿对象
        HSSFWorkbook workbook = new HSSFWorkbook();
        try {
            HSSFSheet sheet = workbook.createSheet("仓库温湿度观测记录表");                  // 创建工作表
            HSSFCellStyle style = getStyle(workbook);

            /*
            * 创建第一行(标题行)
            * */
            HSSFRow row1 = sheet.createRow((short) 0);
            HSSFCell cell1 = null;
            //设置单元格格式
            HSSFCellStyle rowStyle1 = getTitleStyle(workbook);
            //循环创建列
            for (int n = 0; n < 6; n++) {
                cell1 = row1.createCell(n);
                //设置列的格式
                cell1.setCellStyle(rowStyle1);
                //将标题添加到列中
                cell1.setCellValue(new HSSFRichTextString("仓库温湿度观测记录表"));
            }
            //设置行高
            row1.setHeight((short) (30 * 20));
            //合并单元格（参数1：起始行，参数2：截至行，参数3：起始列，参数4：截至列）
            sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 5));


            /*
            * 创建第二行
            * */
            HSSFRow row2 = sheet.createRow((short) 1);
            HSSFCell cell2 = null;
            HSSFCellStyle rowStyle2 = getStyle(workbook);
            cell2 = row2.createCell(0);
            cell2.setCellValue("年度");
            cell2.setCellStyle(rowStyle2);
            cell2 = row2.createCell(1);
            cell2.setCellValue("");
            cell2.setCellStyle(rowStyle2);
            for (int n = 2; n < 6; n++) {
                cell2 = row2.createCell(n);
                //设置列的格式
                cell2.setCellStyle(rowStyle2);
                //将标题添加到列中
                cell2.setCellValue("库房适宜温度为xx℃-xx℃，湿度保持xx%-xx%,超标要做必要调节。");
                sheet.addMergedRegion(new CellRangeAddress(1, 1, 2, 4));
            }
            cell2 = row2.createCell(5);
            cell2.setCellValue("记录人："+userName);
            cell2.setCellStyle(rowStyle2);
            row2.setHeight((short) (26 * 20));

            /*
            * 创建第三行
            * */
            HSSFRow row3 = sheet.createRow((short) 2);
            HSSFCell cell3 = null;
            HSSFCellStyle rowStyle3 = getStyle(workbook);
            for (int n = 0; n < 3; n++) {
                cell3 = row3.createCell(n);
                cell3.setCellStyle(rowStyle3);
                //将标题添加到列中
                cell3.setCellValue("测定时间");
                sheet.addMergedRegion(new CellRangeAddress(2, 2, 0, 2));
            }
            cell3 = row3.createCell(3);
            cell3.setCellValue("天气情况");
            cell3.setCellStyle(rowStyle3);
            cell3 = row3.createCell(4);
            cell3.setCellValue("温度");
            cell3.setCellStyle(rowStyle3);
            cell3 = row3.createCell(5);
            cell3.setCellValue("相对湿度");
            cell3.setCellStyle(rowStyle3);
            row3.setHeight((short) (26 * 20));


            /** 创建第四行
             * */
            HSSFRow row4= sheet.createRow((short) 3);
            HSSFCell cell4 = null;
            HSSFCellStyle rowStyle4 = getStyle(workbook);
            cell4 = row4.createCell(0);
            cell4.setCellValue("月");
            cell4.setCellStyle(rowStyle4);
            cell4 = row4.createCell(1);
            cell4.setCellValue("日");
            cell4.setCellStyle(rowStyle4);
            cell4 = row4.createCell(2);
            cell4.setCellValue("时分");
            cell4.setCellStyle(rowStyle4);
            cell4 = row4.createCell(3);
            cell4.setCellValue("天气情况");
            cell4.setCellStyle(rowStyle4);
            cell4 = row4.createCell(4);
            cell4.setCellValue("温度");
            cell4.setCellStyle(rowStyle4);
            cell4 = row4.createCell(5);
            cell4.setCellValue("相对湿度");
            cell4.setCellStyle(rowStyle4);
            sheet.addMergedRegion(new CellRangeAddress(2, 3, 3, 3));
            sheet.addMergedRegion(new CellRangeAddress(2, 3, 4, 4));
            sheet.addMergedRegion(new CellRangeAddress(2, 3, 5, 5));
            row4.setHeight((short) (26 * 20));

            /** 循环创建维修表的更换部件
             * */
            HSSFCellStyle rowStyle = getStyle(workbook);
            int a=0;
            for(int i = 0;i<warehouseDetailVo.getGatewayOnLineVos().size();i++) {
                for (int j = 0; j < warehouseDetailVo.getGatewayOnLineVos().get(i).getTdlDeviceDetailVos().size(); j++) {
                    HSSFRow row = sheet.createRow((short) 4 + i);
                    HSSFCell cell = null;
                    Long time=Long.valueOf(warehouseDetailVo.getGatewayOnLineVos().get(i).getTdlDeviceDetailVos().get(j).getTime());
//                    SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Date date=new Date(time);
                    cell2 = row2.createCell(1);
                    cell2.setCellValue(date.getYear());
                    cell2.setCellStyle(rowStyle);

                    cell = row.createCell(0);
                    cell.setCellValue(date.getMonth());
                    cell.setCellStyle(rowStyle);
                    cell = row.createCell(1);
                    cell.setCellValue(date.getDay());
                    cell.setCellStyle(rowStyle);
                    cell = row.createCell(2);
                    cell.setCellValue(date.getHours()+":"+date.getMinutes());
                    cell.setCellStyle(rowStyle);
                    cell = row.createCell(3);
                    cell.setCellValue("-");
                    cell.setCellStyle(rowStyle);
                    cell = row.createCell(4);
                    cell.setCellValue(warehouseDetailVo.getGatewayOnLineVos().get(i).getTdlDeviceDetailVos().get(j).getTemperature());
                    cell.setCellStyle(rowStyle);
                    cell = row.createCell(5);
                    cell.setCellValue(warehouseDetailVo.getGatewayOnLineVos().get(i).getTdlDeviceDetailVos().get(j).getHumidity());
                    cell.setCellStyle(rowStyle);
                    row.setHeight((short) (26 * 20));
                }
            }
            sheet.setColumnWidth(0, 10 * 256);
            sheet.setColumnWidth(1, 10 * 256);
            sheet.setColumnWidth(2, 10 * 256);
            sheet.setColumnWidth(3, 27 * 256);
            sheet.setColumnWidth(4, 27 * 256);
            sheet.setColumnWidth(5, 27 * 256);
            return workbook;
        }catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
        }
        return workbook;
    }

    /*
   * 标题单元格样式
   */
    public static HSSFCellStyle getTitleStyle(HSSFWorkbook workbook) {
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints((short)18);
        font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        font.setFontName("微软雅黑");
        HSSFCellStyle style = workbook.createCellStyle();
        style.setFont(font);
        style.setWrapText(true);
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    /*
     * 列数据信息单元格样式
     */
    public static HSSFCellStyle getStyle(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }



    //文字左对齐
    public static HSSFCellStyle getStageAlign(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_LEFT);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_TOP);
        return style;
    }

    //右对齐
    public static HSSFCellStyle getJustifyRight(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为右对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_RIGHT);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    //左对齐
    public static HSSFCellStyle getJustifyLeft(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为右对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_LEFT);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    //左对齐有右边框
    public static HSSFCellStyle getAlignLeft(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_LEFT);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    //左对齐，没有左右边框
    public static HSSFCellStyle getJustifyCente(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_LEFT);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        return style;
    }

    public static HSSFCellStyle getBackGroundStyle(HSSFWorkbook workbook){
        // 设置字体
        HSSFFont font = workbook.createFont();
        //设置字体大小
        font.setFontHeightInPoints((short)11);
        font.setFontName("Arial");
        //设置样式
        HSSFCellStyle style = workbook.createCellStyle();
        style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        style.setBottomBorderColor(HSSFColor.BLACK.index);
        style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        style.setLeftBorderColor(HSSFColor.BLACK.index);
        style.setBorderRight(HSSFCellStyle.BORDER_THIN);
        style.setRightBorderColor(HSSFColor.BLACK.index);
        //设置顶边框;
        style.setBorderTop(HSSFCellStyle.BORDER_THIN);
        style.setTopBorderColor(HSSFColor.BLACK.index);
        //在样式用应用设置的字体;
        style.setFont(font);
        //设置自动换行;
        style.setWrapText(true);
        //设置水平对齐的样式为居中对齐;
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        style.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style.setFillPattern(CellStyle.SOLID_FOREGROUND);
        return style;
    }

}
