package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.OSSClient;
import com.example.tdl.common.WarehouseExcel;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.shiro.SecurityUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/download")
public class DownloadController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Gson gson=new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private TDLLogService tdlLogService;

    private final static String database ="original";

    @ApiOperation(value = "仓库数据下载",notes = "仓库数据下载")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/download",method = RequestMethod.POST)
    public String downLoad(HttpServletResponse response,HttpServletRequest request,@RequestBody String warehouseNo) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        if (StringUtils.isEmpty(warehouseNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(warehouseNo) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        WarehouseDetailVo warehouseDetailVo=warehouseService.getWarehouseDetailByNo(warehouseNo);
        //获取
        List<WarehouseLogVo>  list =tdlLogService.getByWarehouse(warehouseNo);
        List<GatewayOnLineVo> gatewayOnLineVos=new ArrayList<>();
        for (int a=0;a<list.size();a++){
            if(list.get(a) == null){
                continue;
            }
            if(list.get(a).getEndTime()!=null){
                continue;
            }
            GatewayOnLineVo gatewayOnLineVo = new GatewayOnLineVo();
            gatewayOnLineVo.setSN(list.get(a).getSN());
            gatewayOnLineVo.setType(list.get(a).getType());
            Object data = infoRedisService.getHash("DeviceOnline", list.get(a).getDevice() );
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 1){
                    gatewayOnLineVo.setState(7);

                }else {
                    gatewayOnLineVo.setState(6);
                }
            }else{
                gatewayOnLineVo.setState(6);
            }
            String sql ="SELECT \"battery\" FROM \"tdl_policy\".\""+list.get(a).getDevice() +"\" ORDER BY time DESC LIMIT 1";
            QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
            if (queryResult.getResults().get(0).getSeries() != null) {
                if(Double.valueOf(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString())>0){
                    gatewayOnLineVo.setBattery(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1)==null ? "NA" :queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                }else{
                    gatewayOnLineVo.setBattery("NA");
                }
            }else {
                gatewayOnLineVo.setBattery("NA");
            }
            List<String> tdls=tdlDeviceService.getByGatewaySN(list.get(a).getSN(),list.get(a).getType());
            List<TDLDeviceDetailVo> tdlDeviceDetailVos=new ArrayList<>();
            for (int b=0;b<tdls.size();b++){
                TDLDeviceDetailVo tdlDeviceDetailVo=new TDLDeviceDetailVo();
                tdlDeviceDetailVo.setGatewaySN(list.get(a).getSN());
                String sqls ="SELECT \"b\", \"T\", \"h\" FROM \"tdl_policy\".\""+list.get(a).getDevice() +"\" WHERE (\"tdl\" = '"+tdls.get(b)+"') ORDER BY time DESC LIMIT 1";
                QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
                if (queryResults.getResults().get(0).getSeries() != null) {
                    tdlDeviceDetailVo.setTime(parseTime(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                    tdlDeviceDetailVo.setPressure(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                    tdlDeviceDetailVo.setTemperature(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    tdlDeviceDetailVo.setHumidity(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(3).toString());
                }else{
                    tdlDeviceDetailVo.setTime("0");
                    tdlDeviceDetailVo.setPressure("0");
                    tdlDeviceDetailVo.setTemperature("0");
                    tdlDeviceDetailVo.setHumidity("0");
                }
                tdlDeviceDetailVo.setTDLSN(tdls.get(b).replace("TDL-","").trim());
                tdlDeviceDetailVos.add(tdlDeviceDetailVo);
            }
            gatewayOnLineVo.setTdlDeviceDetailVos(tdlDeviceDetailVos);
            gatewayOnLineVos.add(gatewayOnLineVo);
        }
        warehouseDetailVo.setGatewayOnLineVos(gatewayOnLineVos);
        try {
            String fileName = warehouseNo+"仓库温湿度观测记录表-" +System.currentTimeMillis();
            WarehouseExcel warehouseExcel = new WarehouseExcel(fileName,warehouseDetailVo,user.getUserName(),response);
            File file=new File("E:"+File.separator+fileName);
            if(!file.exists()){
                file.mkdir();
            }
            HSSFWorkbook wb = warehouseExcel.downloadExcel();
            if(wb !=null){
                request.setCharacterEncoding("utf-8");
                response.setCharacterEncoding("utf-8");
                response.setContentType("application/vnd.ms-excel");
                response.setHeader("Content-Disposition", "attachment; filename="+fileName);
                OutputStream os = new FileOutputStream(new File(file+".xls"));
                wb.write(os);
                os.flush();
                os.close();
                ZipCompressUtil zipCom = new ZipCompressUtil(file+".zip",file+".xls");
                try{
                    zipCom.zip();
                }catch(Exception e){
                    e.printStackTrace();
                    logger.info(e.toString());
                }
                try {
//                    OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//                    AliyunOSSClientUtil.uploadObject2OSS(ossClient, new File(file +".zip"));
                    if (new File(file+".xls").delete()){
//                        new File(file+".zip").delete();
                        fb.setCode(1);
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/warehouse/" +fileName+".zip");
                        return  gson.toJson(fb);
                    }else{
                        fb.setCode(0);
                        fb.setMessage("下载失败");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info(e.toString());
                    fb.setCode(0);
                    fb.setMessage("上传失败");
                }
            }else{
                fb.setCode(0);
                fb.setMessage("上传失败");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("上传失败");
        }
        return  gson.toJson(fb);
    }

    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }

}
