package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;

import com.example.tdl.domain.vo.AddOrUpdatePermissionVo;
import com.example.tdl.domain.vo.PermissionGroupVo;
import com.example.tdl.domain.vo.PermissionShowVo;
import com.example.tdl.domain.vo.UserRedisVo;
import com.example.tdl.service.PermissionService;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/permission")
public class PermissionController {

    private Logger logger = LoggerFactory.getLogger(PermissionController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private PermissionService permissionService=new PermissionService();

    @Autowired
    private RoleHasPermissionService roleHasPermissionService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;


    //查询本公司的用户
    @ApiOperation(value = "查询所有权限信息",notes = "查询所有权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        //获取该公司的所有权限
        String name = "管理员";
        List<String> permissionLists = roleHasPermissionService.getByRoleName(name);
        List<String> parentLists =permissionService.getPermissionParentName();
        List<PermissionGroupVo> vo =new ArrayList<>();
        //遍历取出每一个action匹配权限组
        for (String parentName: parentLists){
            List<String> permissions =  permissionService.getPermissionByPermissionName(parentName);
            PermissionGroupVo permissionNameGroupVo = new PermissionGroupVo();
            List<PermissionShowVo> permissionShowVoList = new ArrayList<>();
            for(String permissionName:permissions){
                if(permissionLists.contains(permissionName)){
                    PermissionShowVo permissionShowVo =new PermissionShowVo();
                    permissionShowVo.setPermissionName(permissionName);
                    permissionShowVo.setChecked(false);
                    permissionShowVoList.add(permissionShowVo);
                    permissionNameGroupVo.setParentName(parentName);
                }
            }
            permissionNameGroupVo.setPermissionShowVoList(permissionShowVoList);
            if(permissionShowVoList.size() >0){
                vo.add(permissionNameGroupVo);
            }
        }
        return vo;
    }

    //根据角色获取所有的权限
    @ApiOperation(value = "根据角色获取所有的权限",notes = "获取该公司最高权限")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/getPermissionByRoleName",method = RequestMethod.POST)
    public Object getPermissionByRoleName(@RequestBody String roleName,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject json = JSON.parseObject(roleName);
        roleName= (String) json.get("roleName");
        if (roleName.equals("")||roleName==null){
            fb.setCode(0);
            fb.setMessage("查询的角色名不能为空");
            return gson.toJson(fb);
        }

        //获取该公司的所有权限
        String name = "管理员";
        List<String> permissionLists = roleHasPermissionService.getByRoleName(name);
        List<String> parentLists =permissionService.getPermissionParentName();
        //根据角色名获取拥有的角色
        List<String> rolePermission = roleHasPermissionService.getByRoleName(roleName);
        List<PermissionGroupVo> vo =new ArrayList<>();
        //遍历取出每一个action匹配权限组
        for (String parentName : parentLists) {
            List<String> permissions = permissionService.getPermissionByPermissionName(parentName);
            PermissionGroupVo permissionGroupVo = new PermissionGroupVo();
            permissionGroupVo.setParentName(parentName);
            List<PermissionShowVo> permissionShowVosList = new ArrayList<>();
            for (String permission : permissions) {
                PermissionShowVo permissionShowVo = new PermissionShowVo();
                if (permissionLists.contains(permission)) {
                    permissionShowVo.setPermissionName(permission);
                    if (rolePermission.contains(permission)) {
                        permissionShowVo.setChecked(true);
                    } else {
                        permissionShowVo.setChecked(false);
                    }
                    permissionShowVosList.add(permissionShowVo);
                }
            }
            permissionGroupVo.setPermissionShowVoList(permissionShowVosList);
            if (permissionShowVosList.size() > 0) {
                vo.add(permissionGroupVo);
            }
        }
        return vo;
    }


    //新增或者修改权限
    @ApiOperation(value = "添加或修改权限", notes = "根据角色添加或修改权限")
    @RequestMapping(value = "/addOrSavePermission",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String addOrSavePermission(@RequestBody AddOrUpdatePermissionVo addOrUpdatePermissionVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addOrUpdatePermissionVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"roleName"));
            return gson.toJson(fb);
        }
        String permissionVar="";
        int num = 0;
        for(int i=0;i<addOrUpdatePermissionVo.getPermissionGroupVos().size();i++){
            permissionVar+=addOrUpdatePermissionVo.getPermissionGroupVos().get(i)+" ";
        }
        try {
            Map<String,Object> map=new HashMap<>();
            map.put("roleName",addOrUpdatePermissionVo.getRoleName());
            map.put("companyName",user.getCompanyName());
            map.put("permissionName",permissionVar);
            map.put("num",addOrUpdatePermissionVo.getPermissionGroupVos().size());
            roleHasPermissionService.addRoleHasPermission(map);
            Map<String,Object> ms=new HashMap<>();
            ms.put("msg",map.get("msg"));
            if (ms.get("msg").equals("权限操作成功")){
                fb.setCode(1);
                fb.setMessage(ms.get("msg").toString());
            }else {
                fb.setCode(0);
                fb.setMessage(ms.get("msg").toString());
            }
        } catch (DataAccessException e) {
            fb.setCode(0);
            fb.setMessage("操作失败");
        }
        return gson.toJson(fb);
    }

}
